//--------------------------------------------------------------------//
//                                                                      
// Program for logging Dallas one-wire devices 
// connected through a PIC Arbiter on a serial port
//
// Copyright (C) 2005-2006  Vesa Tervo 
// <oh3nwq@sral.fi> OH3NWQ  2005-07-17
// 
// last edited 2006-05-08
//
// Released under the  conditions  of  the  GNU General 
// Public License.
//
// http://www.gnu.org/copyleft/gpl.html
//
// 
//
//    MAY THE SOURCE BE WITH YOU
//
//--------------------------------------------------------------------//


//
// Based on logserial-0.4.2 by Sinkovics Zoltan Copyright (c) 1998,2000
//


// includes CRC algorithm from Colin O'Flynn - Copyright (c) 2002
// only minor changes by M.Thomas 9/2004



#include "logarbiter.h"
#include "crc.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <signal.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <inttypes.h>

#define   MAXPATH   255
#define   CRC8INIT  0x00
#define   CRC8POLY  0x18              //0X18 = X^8+X^5+X^4+X^0

char NAME[MAXPATH]; // program name
int portfd;         // serial port file descriptor
int pidfile=1;
int x=0,debug=0,kill_old=0,logging=0;
char command_line[128];

struct tm *ptr;
time_t tm;
char str[60];

void closetty(int sig) ;
void version(void) ;
void usage(void) ;


uint8_t crc8 ( uint8_t *data_in, uint16_t number_of_bytes_to_read )
{
//This code is from Colin O'Flynn - Copyright (c) 2002
//only minor changes by M.Thomas 9/2004
//
//Permission is hereby granted, free of charge, to any person obtaining acopy of
//this software and associated documentation files (the "Software"), todeal in
//the Software without restriction, including without limitation the rights to
//use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
//the Software, and to permit persons to whom the Software is furnished to do so,
//subject to the following conditions:
//
//The above copyright notice and this permission notice shall be included in all
//copies or substantial portions of the Software.

        uint8_t  crc;
        uint16_t loop_count;
        uint8_t  bit_counter;
        uint8_t  data;
        uint8_t  feedback_bit;

        crc = CRC8INIT;

        for (loop_count = 0; loop_count != number_of_bytes_to_read; loop_count++)
        {
                data = data_in[loop_count];

                bit_counter = 8;
                do {
                        feedback_bit = (crc ^ data) & 0x01;

                        if ( feedback_bit == 0x01 ) {
                                crc = crc ^ CRC8POLY;
                        }
                        crc = (crc >> 1) & 0x7F;
                        if ( feedback_bit == 0x01 ) {
                                crc = crc | 0x80;
                        }

                        data = data >> 1;
                        bit_counter--;

                } while (bit_counter > 0);
        }

        return crc;
}


void version(void)
{
    printf("\n%s - %s (C) 2005-2006 Vesa Tervo OH3NWQ\n\n",NAME,VERSION);
}

void usage(void)
{
    version();
    printf(" usage: %s [parameters]\n\n",NAME);
    printf("\tOptional parameters are:\n");
    printf("\t\t -h prints this help screen\n");
    printf("\t\t -v prints version of %s\n",NAME);
    printf("\t\t -D run as a daemon\n");
    printf("\t\t -t tty                                default /dev/ttyS0\n");
    printf("\t\t -s speed                              default 19200\n");
    printf("\t\t -b data bits                          default 8\n");
    printf("\t\t -p parity                             default N\n");
    printf("\t\t -f Fahrenheit                         default Celsius\n");
    printf("\t\t -V verbose                            default mute\n");
    printf("\t\t -l enable error logging to /dev/tty11\n");
    printf("\t\t    and /var/log/logarbiter            default disbled\n");
    printf("\t\t -u shows the unit                     default not shown\n");
    printf("\t\t -d debug                              default not shown\n");
    printf("\t\t -c data save directory                default .\n");  
    printf("\t\t -P don't write lock file              default /var/run/logarbiter.pid\n");
    printf("\t\t -x number of sensors                  default 1\n");
    printf("\t\t -K kill the process in /var/run/logarbiter.pid\n\n");
    printf("\tExamples:    %s -V -u -l\n",NAME);
    printf("\t             %s -D -c ~www/htdocs \n\n",NAME);   
}

int ctoi(char c)
{
  switch(c) {
  case '0': return(0);
  case '1': return(1);
  case '2': return(2);
  case '3': return(3);
  case '4': return(4);
  case '5': return(5);
  case '6': return(6);
  case '7': return(7);
  case '8': return(8);
  case '9': return(9);
  case 'a':
  case 'A': return(10);
  case 'b':
  case 'B': return(11);
  case 'c':
  case 'C': return(12);
  case 'd':
  case 'D': return(13);
  case 'e':
  case 'E': return(14);
  case 'f':
  case 'F': return(15);
  default : return(65535);  
  }
}

int main(int argc, char **argv)
{
    int c,childproc;      
    char tty[MAXPATH];      // serial port tty
    char outdir[MAXPATH];   // output directory
    char filename[MAXPATH]; // filename for output
    
    char baudrate[10];      // baud rate as a string
    char parity[10];        // parity as a string
    char bits[10];          // data bits as a string
    int hwhandshake=1;      // true if RTS/CTS handshake used
    int swhandshake=0;      // true if XON/XOFF handshake used
    int n;                  // just for the for

    int verbose=0;
    int daemon=0; 

    char crc1=0xff,crc2=0xff; 

    mode_t rwxmode;         // read,write,execute rights
    
    int loop=0;             // used to bail out before the sun sets 
    int max_loop=4;         // maximum time in the main loop b4 next measurement

    int y=0,z=0,counter=0;  // parameters used to calculate the temperature     
    float temperature=-273.16;  // Temperature in Celsius 

    int imperial=0;         // use funny imperial units 
    int unit=0;             // show C or F after the temperature value  
    int old_pid=-1;
    
    fd_set fds;             // file descriptor set for reading
    struct timeval tv;      // struct for time interval for select
    int buflen ;            // length of buffer
    char buf[128] ;         // input character buffer
    int sensors=1, shown=0;
    int continuous=0;
    char sensor_buffer[128]; // buffer to read the current sensor data    
    char sensor_address[6]; // address of current sensor; 
    int sensor_type=0;
  
    char crc_buffer[9]; 

    FILE *output_file;
    FILE *InputFile;

    strcpy(bits         , "8"   );
    strcpy(parity       , "N"   );
    strcpy(baudrate     , "19200" );
    strcpy(tty          , "/dev/ttyS0"  );
    strcpy(NAME         , "logarbiter"  );
    strcpy(outdir       , "."   );
    strcpy(sensor_buffer, "default data" );
    
    rwxmode = S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH ;
    
    /* parse command line */
    while ((c = getopt (argc, argv, "t:s:b:p:ufvhdx:c:VDPKl")) != EOF)
    {
     switch (c)
         {
         case 'h':       // help
          usage();
          exit(EXIT_SUCCESS);
         break;
         case 'v':       // version
          version();
          exit(EXIT_SUCCESS);
         break;
         case 't':       // tty
          if (strlen(optarg) > (MAXPATH-1))
           {
           printf("Too long tty path argument\n");
           exit(EXIT_FAILURE);
           }
          strcpy(tty, optarg);
         break;
	 case 'l':
	   logging = 1;
	 break;
         case 'c':                           // output directory
          if (strlen(optarg) > (MAXPATH-1))
           {
           printf("Too long tty path argument\n");
           exit(EXIT_FAILURE);
           }
           strcpy(outdir, optarg);
          max_loop=16;    //makes loop forever
          continuous = 1;
         break;
         case 's':       // speed
          if (strlen(optarg)>9)
           {
           printf("Invalid baudrate argument\n");
           exit(EXIT_FAILURE);
           }
          strcpy (baudrate, optarg);
         break;
         case 'b':       // data
         if (strlen(optarg)>9)
           {
           printf("Invalid bit number argument\n");
           exit(EXIT_FAILURE);
           }
          strcpy (bits, optarg);
         break;
         case 'p':       // parity
          if (strlen(optarg)>9)
           {
           printf("Invalid parity argument\n");
           exit(EXIT_FAILURE);
           }
          strcpy (parity, optarg);
         break;
         case 'D':                           // Daemon
            daemon=1;
         break;
         case 'V':                           // Verbose
            verbose=1;
         break;
         case 'P':                           // pid file checking off
            pidfile=0;
         break;
         case 'd':                           // raw data
            debug=1;
         break;
         case 'u':       // C or F shown
            unit=1;
         break;
         case 'f':                           // Fahrenheit
            imperial=1;
         break;
         case 'K':                           // kill
            kill_old=1;
         break;
           case 'x':       // number of sensors
          sensors = ctoi(optarg[0]);
          if (strlen(optarg)>1)        
             { 
              printf("Invalid sensor number argument\n");        
              exit(EXIT_FAILURE);        } 
       
          if (sensors < 1)
             {
              printf("Invalid sensor number argument\n");
              exit(EXIT_FAILURE);    }
       
          //if (debug) printf("number of sensors = %d\n",sensors);
        break;
        }
    }

    // kill daemon if requested...

    if (kill_old) {
      // open /var/run/logarbiter.pid
      // read the pid
      
      InputFile = fopen("/var/run/logarbiter.pid", "r");
      if (InputFile == NULL) {
          printf("*** error - pid file not found\n");
          exit(-3);
          }
  
        fscanf(InputFile, "%d", &old_pid);
        fclose(InputFile);
      
      // kill the process
      x = sprintf (command_line, "kill %d",old_pid);
      if (debug) printf ("command line to execute: %s\n", command_line); 
      x = system(command_line);
          
      x = sprintf (command_line, "rm -f /var/run/logarbiter.pid");
      if (debug) printf ("command line to execute: %s\n", command_line); 
      x = system(command_line);

      if (logging == 1)
      { 	

      tm = time(NULL);
      ptr = localtime(&tm);
      strftime(str ,100 , "%d.%m.%Y %H:%M:%S",ptr);

//      x = sprintf (command_line, "date  > /dev/tty11");
//      if (debug)  printf ("command line to execute: %s\n", command_line);
//      x = system(command_line);

//      x = sprintf (command_line, "date  >> /var/log/logarbiter.log");
//      if (debug)  printf ("command line to execute: %s\n", command_line);
//      x = system(command_line);

      x = sprintf (command_line, "echo -e \"%s *** logarbiter exiting\"  > /dev/tty11",str);
      if (debug)  printf ("command line to execute: %s\n", command_line);
      x = system(command_line);

      x = sprintf (command_line, "echo -e \"%s *** logarbiter exiting\"  >> /var/log/logarbiter.log",str);
      if (debug)  printf ("command line to execute: %s\n", command_line);
      x = system(command_line);
      }

      exit(EXIT_SUCCESS); 
    }
  
    InputFile = fopen("/var/run/logarbiter.pid", "r");
    if (InputFile != NULL) {
       fscanf(InputFile, "%d", &old_pid);
       fclose(InputFile);
       printf("*** error - %s process %d is already running\n",NAME,old_pid);
       exit(EXIT_FAILURE);
       }

  // daemonize now if requested...

        if(daemon==1) {
                childproc=fork();
                if(childproc==-1) {
                        perror("logarbiter - Unable to fork");
                        exit(EXIT_FAILURE);
                }
                if(childproc!=0) {
                        fprintf(stderr,"Started logarbiter daemon...\n");
			
			if (logging == 1)
			{

      			tm = time(NULL);
      			ptr = localtime(&tm);
      			strftime(str ,100 , "%d.%m.%Y %H:%M:%S",ptr);

	//		x = sprintf (command_line, "date  > /dev/tty11");
     	//		if (debug)  printf ("command line to execute: %s\n", command_line);
      	//		x = system(command_line);

        //              x = sprintf (command_line, "date  >> /var/log/logarbiter.log");
        //              if (debug)  printf ("command line to execute: %s\n", command_line);
        //              x = system(command_line);

      			x = sprintf (command_line, "echo -e \"%s Started logarbiter daemon...\"  > /dev/tty11",str);
      			if (debug)  printf ("command line to execute: %s\n", command_line);
     		 	x = system(command_line);

                        x = sprintf (command_line, "echo -e \"%s Started logarbiter daemon...\"  >> /var/log/logarbiter.log",str);
                        if (debug)  printf ("command line to execute: %s\n", command_line);
                        x = system(command_line);
			}

                        exit(EXIT_SUCCESS);
                }
        }

        portfd = open(tty,O_RDWR);
    
    if (portfd == -1) 
      {
      printf("Can't open %s .\n",tty);
      exit(EXIT_FAILURE);
      }

    m_savestate(portfd);
    m_setparms(portfd,baudrate,parity,bits,hwhandshake,swhandshake);
    m_nohang(portfd);
    m_hupcl(portfd,1);
    m_flush(portfd);

//*    // Stow our tty name in a global variable
//*    // so we can remove the lock when the program exits.
//*    strcpy(myTTY, tty) ;

    signal (SIGHUP, &closetty);
    signal (SIGINT, &closetty);
    signal (SIGQUIT, &closetty);
    signal (SIGTERM, &closetty);
    signal (SIGABRT, &closetty);

    if (pidfile)     {
      if (debug) printf("Pid: %d\n",getpid());
      x = sprintf (command_line, "echo -e \"%d\" > /var/run/logarbiter.pid",getpid());
      if (debug)  printf ("command line to execute: %s\n", command_line);
      x = system(command_line);
    }

    while(1)
    {     
    if (loop > max_loop)      // 8 sensors less than 5 sec
    {
      if (logging == 1)
      {
      tm = time(NULL);
      ptr = localtime(&tm);
      strftime(str ,100 , "%d.%m.%Y %H:%M:%S",ptr);

      // x = sprintf (command_line, "date  > /dev/tty11");
      // if (debug)  printf ("command line to execute: %s\n", command_line);
      // x = system(command_line);

      // x = sprintf (command_line, "date  >> /var/log/logarbiter.log");
      // if (debug)  printf ("command line to execute: %s\n", command_line);
      // x = system(command_line);

      x = sprintf (command_line, "echo -e \"%s *** timeout\"  > /dev/tty11",str);
      if (debug)  printf ("command line to execute: %s\n", command_line);
      x = system(command_line);

      x = sprintf (command_line, "echo -e \"%s *** timeout\"  >> /var/log/logarbiter.log",str);
      if (debug)  printf ("command line to execute: %s\n", command_line);
      x = system(command_line);
      }

      //printf("*** timeout\n"); 
      exit(EXIT_FAILURE);
    }
    loop++;

      tv.tv_sec=1  ;      // 1 second
      tv.tv_usec=0 ;
      
      FD_ZERO(&fds );
      FD_SET(portfd, &fds);
      
      if (select(portfd+1, &fds, NULL, NULL, &tv) > 0)
        {
    buflen = read(portfd, buf, 127);
    if (buflen == -1) closetty(-1);

    if (buflen > 0 )        // got input bytes
      {
      loop = 0;

      for(n=0;n < buflen;n++)
      {
      counter++;
      
      sensor_buffer[counter-1] = buf[n];

      if (debug) printf("%c",buf[n]);
      
      sensor_type = ctoi(sensor_buffer[0])*16 + ctoi(sensor_buffer[1]); 
        
      if (buf[n] == 0x0a)
        {

      x = sprintf (crc_buffer, "%c%c%c%c%c%c%c%c",
         ctoi(sensor_buffer[0*3+0])*16 + ctoi(sensor_buffer[0*3+1]),
         ctoi(sensor_buffer[1*3+0])*16 + ctoi(sensor_buffer[1*3+1]),
         ctoi(sensor_buffer[2*3+0])*16 + ctoi(sensor_buffer[2*3+1]),
         ctoi(sensor_buffer[3*3+0])*16 + ctoi(sensor_buffer[3*3+1]),
         ctoi(sensor_buffer[4*3+0])*16 + ctoi(sensor_buffer[4*3+1]),
         ctoi(sensor_buffer[5*3+0])*16 + ctoi(sensor_buffer[5*3+1]),
         ctoi(sensor_buffer[6*3+0])*16 + ctoi(sensor_buffer[6*3+1]),
         ctoi(sensor_buffer[7*3+0])*16 + ctoi(sensor_buffer[7*3+1]));
    
      crc1 = crc8(crc_buffer,8);    
      if (debug) printf("crc 1= [%d]\n",crc1);

      x = sprintf (crc_buffer, "%c%c%c%c%c%c%c%c%c",
         ctoi(sensor_buffer[8*3+0])*16 + ctoi(sensor_buffer[8*3+1]),
         ctoi(sensor_buffer[9*3+0])*16 + ctoi(sensor_buffer[9*3+1]),
         ctoi(sensor_buffer[10*3+0])*16 + ctoi(sensor_buffer[10*3+1]),
         ctoi(sensor_buffer[11*3+0])*16 + ctoi(sensor_buffer[11*3+1]),
         ctoi(sensor_buffer[12*3+0])*16 + ctoi(sensor_buffer[12*3+1]),
         ctoi(sensor_buffer[13*3+0])*16 + ctoi(sensor_buffer[13*3+1]),
         ctoi(sensor_buffer[14*3+0])*16 + ctoi(sensor_buffer[14*3+1]),
         ctoi(sensor_buffer[15*3+0])*16 + ctoi(sensor_buffer[15*3+1]),
         ctoi(sensor_buffer[16*3+0])*16 + ctoi(sensor_buffer[16*3+1]));

      crc2 = crc8(crc_buffer,9);
      if (debug) printf("crc 2= [%d]\n",crc2);

    if ((crc1 == 0) && (crc2 == 0)) {   
      switch (sensor_type)
      {
      case (0x10):              
        // if (sensor_type == 0x10)
          
        if (debug) printf("Sensor is DS18S20 or DS1920\n");
              x = ctoi(sensor_buffer[24])*16 + ctoi(sensor_buffer[25]);
              y = ctoi(sensor_buffer[27])*16 + ctoi(sensor_buffer[28]);
              z = ctoi(sensor_buffer[42])*16 + ctoi(sensor_buffer[43]);
		x = x>>1;

		temperature = (x*1.0)+((15-z)/16.0)+0.25;

          if (y==0xff)
          {
	    temperature = temperature-128.0;
          } 
          break;

      case (0x22):
                           // if (sensor_type == 0x22)
                            
                                if (debug) printf("Sensor is DS1822\n");
                                x=ctoi(sensor_buffer[24])*16 + ctoi(sensor_buffer[25]);
                                y=ctoi(sensor_buffer[27])*16 + ctoi(sensor_buffer[28]);
                                z=ctoi(sensor_buffer[27]);

                                temperature = ((y<<8)+x)*0.0625;

                                if (z == 0xf) temperature = temperature - 65536*0.0625;
                            break;

      case (0x28):    
         // if (sensor_type == 0x28)
          
                                if (debug) printf("Sensor is DS18B20\n"); 
                                x=ctoi(sensor_buffer[24])*16 + ctoi(sensor_buffer[25]);
        			y=ctoi(sensor_buffer[27])*16 + ctoi(sensor_buffer[28]);
        			z=ctoi(sensor_buffer[27]);

                                temperature = ((y<<8)+x)*0.0625;
 
        if (z == 0xf) temperature = temperature - 65536*0.0625;
          break;

      default:
        if (debug) printf("***Error - unknown sensor type: %X\n",sensor_type);
      }

      //printf ("\t Temperature is: %6.2f C\n", temperature);

      if (imperial) temperature = (temperature*9.0/5.0)+32.0;
             
      x = strlen(sensor_buffer);

//         printf ("length of sensor buffer is %d \n", x);
//         printf ("Sensor buffer is %s \n", sensor_buffer);

      x = sprintf(sensor_address,"%c%c%c%c%c%c%c%c%c%c%c%c", 
                                     sensor_buffer[18],sensor_buffer[19],
                                     sensor_buffer[15],sensor_buffer[16],
                                     sensor_buffer[12],sensor_buffer[13],
                                     sensor_buffer[9],sensor_buffer[10],
                                     sensor_buffer[6],sensor_buffer[7],
                                     sensor_buffer[3],sensor_buffer[4]);
 
      if (debug) printf ("Received data from sensor: %s\n",sensor_address);
        
      x = sprintf(filename, "%s/%s",outdir,sensor_address);
      
      if ((output_file = fopen(filename, "w")) == NULL)
               exit(-2);       

      fprintf(output_file,"%-.2f\n", temperature);

      fclose(output_file);  

      if (debug) printf ("Sensor data (%-.2f) saved to: %s\n",temperature,filename);

//  as the sensor IDs are unique, this does not hurt

//  my outdoor sensor address is 000800C61533

      y = strcmp(sensor_address, "000800C61533");       
      if (y == 0)
        {

          // temperature = temperature + 0.5;      //calibration value

          if (debug) printf ("Sensor address matches\n");
          x = sprintf (command_line, "/usr/bin/rrdtool update /home/oh3nwq/rrd/outside.rrd N:%-8.4f", temperature);
          if (debug) printf ("command line to execute: %s\n", command_line); 
          x = system(command_line);
        }

//      my indoor sensor address is 0000000F34B2

        y = strcmp(sensor_address, "0000000F34B2");  
        if (y == 0)
        {
          // temperature = temperature + 0.4;    //calibration value

          if (debug) printf ("Sensor address matches\n");
          x = sprintf (command_line, "/usr/bin/rrdtool update /home/oh3nwq/rrd/inside.rrd N:%-8.4f", temperature);
          if (debug) printf ("command line to execute: %s\n", command_line);
          x = system(command_line);
        }

//      another ds18s20 is way off from 0.5C accuracy 000800C6449E

        y = strcmp(sensor_address, "000800C6449E");
        if (y == 0)
        {

          temperature = temperature + 3.0;    //calibration value
        }


                           if ((debug) || (verbose))
                             {
                             printf ("%-8.4f", temperature);

                             if (unit)
                             {
                                if (imperial) printf(" F");
                                else printf(" C");
                             }
                             printf ("\n");
                             }
  }
  else  { 
	//printf ("*** crc error\n"); 
        if (logging == 1)
	{
       tm = time(NULL);
       ptr = localtime(&tm);
       strftime(str ,100 , "%d.%m.%Y %H:%M:%S",ptr);

//        x = sprintf (command_line, "date  > /dev/tty11");
//        if (debug)  printf ("command line to execute: %s\n", command_line);
//        x = system(command_line);

//        x = sprintf (command_line, "date  >> /var/log/logarbiter.log");
//        if (debug)  printf ("command line to execute: %s\n", command_line);
//        x = system(command_line);

        x = sprintf (command_line, "echo %s ***crc error  > /dev/tty11",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);
	
        x = sprintf (command_line, "echo %s ***crc error  >> /var/log/logarbiter.log",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);
	}

	}
    
        strcpy(sensor_buffer, "" );
        counter = 0;
        x = 0;
        y = 0;
        z = 0;
                        
        if (!continuous) shown++;

        if (shown == sensors){ 
          x = sprintf (command_line, "rm -f /var/run/logarbiter.pid");
          if (debug) printf ("command line to execute: %s\n", command_line);
          x = system(command_line);
          exit(EXIT_SUCCESS);}

        }   
        }
        }
       }
      }
    return(0);
}

void closetty(int sig)
{
//*    pid_t temp_lock ;

  tm = time(NULL);
  ptr = localtime(&tm);
  strftime(str ,100 , "%d.%m.%Y %H:%M:%S",ptr);

  switch (sig) {
  case -1:
      printf("%s: %s Read error on tty. Closing communication port\n",NAME,str);
        if (logging == 1)
        {
        x = sprintf (command_line, "echo %s Read error on tty. Closing communication port > /dev/tty11",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);

        x = sprintf (command_line, "echo %s Read error on tty. Closing communication port >> /var/log/logarbiter.log",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);
	} 
      break;
  case -2:
      printf("%s: %s Write error on output. Closing communication port\n",NAME,str);
        if (logging == 1)
        {
        x = sprintf (command_line, "echo %s Write error on tty. Closing communication port > /dev/tty11",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);

        x = sprintf (command_line, "echo %s Write error on tty. Closing communication port >> /var/log/logarbiter.log",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);
        }
      break;
  case -3:
      printf("%s: %s Read error on input file. Closing communication port\n",NAME,str);
        if (logging == 1)
        {
        x = sprintf (command_line, "echo %s Read error on input file. Closing communication port > /dev/tty11",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);

        x = sprintf (command_line, "echo %s Read error on input file. Closing communication port >> /var/log/logarbiter.log",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);
        }
      break;
  case -4:
      printf("%s: %s Write error on tty. Closing communication port\n",NAME,str);
        if (logging == 1)
        {
        x = sprintf (command_line, "echo %s Write error on tty. Closing communication port > /dev/tty11",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);

        x = sprintf (command_line, "echo %s Write error on tty. Closing communication port >> /var/log/logarbiter.log",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);
        }
      break;
  case 15:
      printf("%s: %s Got KILL signal. Closing communication port\n",NAME,str);
        if (logging == 1)
        {
        x = sprintf (command_line, "echo %s Got KILL signal. Closing communication port > /dev/tty11",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);

        x = sprintf (command_line, "echo %s Got KILL signal. Closing communication port >> /var/log/logarbiter.log",str);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);
        }
      break;
  default:
      printf("%s: %s Got signal %d. Closing communication port\n",NAME,str,sig);
        if (logging == 1)
        {
        x = sprintf (command_line, "echo %s Got signal %d. Closing communication port > /dev/tty11",str,sig);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);

        x = sprintf (command_line, "echo %s Got signal %d. Closing communication port >> /var/log/logarbiter.log",str,sig);
        if (debug)  printf ("command line to execute: %s\n", command_line);
        x = system(command_line);
        }

      break;
  }

    m_restorestate(portfd);
    
    close(portfd);

     if (daemon == 0) {
        x = sprintf (command_line, "rm -f /var/run/logarbiter.pid");
        if (debug) printf ("command line to execute: %s\n", command_line);
        x = system(command_line);
        }

    exit(EXIT_SUCCESS);
}
